@@ -2,6 +2,7 @@
 
 #include "cmds.h"
 #include "libbcachefs.h"
+#include "init/fs.h"
 #include "sb/io.h"
 
 static void reset_counters_usage(void)
@@ -40,15 +41,23 @@ int cmd_reset_counters(int argc, char *argv[])
 	if (argc)
 		die("too many arguments");
 
+	/* scan for devices, open full fs */
+
 	struct bch_opts opts = bch2_opts_empty();
-	struct bch_sb_handle sb;
-	int ret = bch2_read_super(dev, &opts, &sb);
+	opt_set(opts, nostart,	true);
+	opt_set(opts, degraded, BCH_DEGRADED_very);
+
+	darray_const_str devs = get_or_split_cmdline_devs(1, &dev);
+
+	struct bch_fs *c = bch2_fs_open(&devs, &opts);
+	int ret = PTR_ERR_OR_ZERO(c);
 	if (ret)
-		die("Error opening %s: %s", dev, bch2_err_str(ret));
+		die("Error opening filesystem: %s", bch2_err_str(ret));
 
-	bch2_sb_field_resize(&sb, counters, 0);
-
-	bch2_super_write(sb.bdev->bd_fd, sb.sb);
-	bch2_free_super(&sb);
+	scoped_guard(mutex, &c->sb_lock) {
+		bch2_sb_field_resize(&c->disk_sb, counters, 0);
+		bch2_write_super(c);
+	}
+	bch2_fs_stop(c);
 	return 0;
 }
