@@ -31,20 +31,22 @@
 
 /* bch_extent_rebalance: */
 
-static const struct bch_extent_rebalance *bch2_bkey_ptrs_rebalance_opts(struct bkey_ptrs_c ptrs)
+static const struct bch_extent_rebalance *bch2_bkey_ptrs_rebalance_opts(const struct bch_fs *c,
+									struct bkey_ptrs_c ptrs)
 {
 	const union bch_extent_entry *entry;
 
 	bkey_extent_entry_for_each(ptrs, entry)
-		if (__extent_entry_type(entry) == BCH_EXTENT_ENTRY_rebalance)
+		if (extent_entry_type(entry) == BCH_EXTENT_ENTRY_rebalance)
 			return &entry->rebalance;
 
 	return NULL;
 }
 
-static const struct bch_extent_rebalance *bch2_bkey_rebalance_opts(struct bkey_s_c k)
+static const struct bch_extent_rebalance *bch2_bkey_rebalance_opts(const struct bch_fs *c,
+								   struct bkey_s_c k)
 {
-	return bch2_bkey_ptrs_rebalance_opts(bch2_bkey_ptrs_c(k));
+	return bch2_bkey_ptrs_rebalance_opts(c, bch2_bkey_ptrs_c(k));
 }
 
 void bch2_extent_rebalance_to_text(struct printbuf *out, struct bch_fs *c,
@@ -135,7 +137,7 @@ static void bch2_bkey_needs_rebalance(struct bch_fs *c, struct bkey_s_c k,
 
 	struct bkey_ptrs_c ptrs = bch2_bkey_ptrs_c(k);
 
-	const struct bch_extent_rebalance *rb_opts = bch2_bkey_ptrs_rebalance_opts(ptrs);
+	const struct bch_extent_rebalance *rb_opts = bch2_bkey_ptrs_rebalance_opts(c, ptrs);
 	if (!io_opts && !rb_opts)
 		return;
 
@@ -236,7 +238,7 @@ int bch2_bkey_set_needs_rebalance(struct bch_fs *c, struct bch_inode_opts *opts,
 
 	struct bkey_s k = bkey_i_to_s(_k);
 	struct bch_extent_rebalance *old =
-		(struct bch_extent_rebalance *) bch2_bkey_rebalance_opts(k.s_c);
+		(struct bch_extent_rebalance *) bch2_bkey_rebalance_opts(c, k.s_c);
 
 	if (bkey_should_have_rb_opts(c, opts, k.s_c)) {
 		if (!old) {
@@ -247,7 +249,7 @@ int bch2_bkey_set_needs_rebalance(struct bch_fs *c, struct bch_inode_opts *opts,
 		*old = io_opts_to_rebalance_opts(c, opts);
 	} else {
 		if (old)
-			extent_entry_drop(k, (union bch_extent_entry *) old);
+			extent_entry_drop(c, k, (union bch_extent_entry *) old);
 	}
 
 	return 0;
@@ -270,7 +272,7 @@ int bch2_update_rebalance_opts(struct btree_trans *trans,
 	if (bkey_is_btree_ptr(k.k))
 		return 0;
 
-	const struct bch_extent_rebalance *old = bch2_bkey_rebalance_opts(k);
+	const struct bch_extent_rebalance *old = bch2_bkey_rebalance_opts(c, k);
 	struct bch_extent_rebalance new = io_opts_to_rebalance_opts(c, io_opts);
 
 	if (bkey_should_have_rb_opts(c, io_opts, k)
@@ -290,13 +292,20 @@ int bch2_bkey_get_io_opts(struct btree_trans *trans,
 			  struct per_snapshot_io_opts *snapshot_opts, struct bkey_s_c k,
 			  struct bch_inode_opts *opts)
 {
+	enum io_opts_mode {
+		IO_OPTS_metadata,
+		IO_OPTS_reflink,
+		IO_OPTS_user,
+	} mode = bkey_is_btree_ptr(k.k) ? IO_OPTS_metadata :
+		 !bkey_is_indirect(k.k)	? IO_OPTS_user :
+					  IO_OPTS_reflink;
+
 	struct bch_fs *c = trans->c;
-	bool metadata = bkey_is_btree_ptr(k.k);
 
 	if (!snapshot_opts) {
-		bch2_inode_opts_get(c, opts, metadata);
+		bch2_inode_opts_get(c, opts, mode == IO_OPTS_metadata);
 
-		if (k.k->p.snapshot) {
+		if (mode == IO_OPTS_user) {
 			struct bch_inode_unpacked inode;
 			int ret = bch2_inode_find_by_inum_snapshot(trans, k.k->p.inode, k.k->p.snapshot,
 								   &inode, BTREE_ITER_cached);
@@ -306,54 +315,62 @@ int bch2_bkey_get_io_opts(struct btree_trans *trans,
 				bch2_inode_opts_get_inode(c, &inode, opts);
 		}
 	} else {
-		if (snapshot_opts->fs_io_opts.change_cookie != atomic_read(&c->opt_change_cookie)) {
+		/*
+		 * If we have a per_snapshot_io_opts, we're doing a scan in
+		 * natural key order: we can cache options for the inode number
+		 * we're currently on, but we have to cache options from every
+		 * different snapshot version of that inode
+		 */
+
+		bool metadata = mode == IO_OPTS_metadata;
+		if (snapshot_opts->fs_io_opts.change_cookie	!= atomic_read(&c->opt_change_cookie) ||
+		    snapshot_opts->metadata			!= metadata) {
 			bch2_inode_opts_get(c, &snapshot_opts->fs_io_opts, metadata);
 
+			snapshot_opts->metadata = metadata;
 			snapshot_opts->cur_inum = 0;
-			snapshot_opts->d.nr = 0;
+			snapshot_opts->d.nr	= 0;
 		}
 
-		if (k.k->p.snapshot) {
+		if (mode == IO_OPTS_user) {
 			if (snapshot_opts->cur_inum != k.k->p.inode) {
 				snapshot_opts->d.nr = 0;
 
-				int ret = for_each_btree_key(trans, iter, BTREE_ID_inodes, POS(0, k.k->p.inode),
-							     BTREE_ITER_all_snapshots, k, ({
-					if (k.k->p.offset != k.k->p.inode)
-						break;
-
-					if (!bkey_is_inode(k.k))
+				try(for_each_btree_key_max(trans, iter, BTREE_ID_inodes,
+							   SPOS(0, k.k->p.inode, 0),
+							   SPOS(0, k.k->p.inode, U32_MAX),
+							   BTREE_ITER_all_snapshots, inode_k, ({
+					struct bch_inode_unpacked inode;
+					if (!bkey_is_inode(inode_k.k) ||
+					    bch2_inode_unpack(inode_k, &inode))
 						continue;
 
-					struct bch_inode_unpacked inode;
-					_ret3 = bch2_inode_unpack(k, &inode);
-					if (_ret3)
-						break;
-
-					struct snapshot_io_opts_entry e = { .snapshot = k.k->p.snapshot };
+					struct snapshot_io_opts_entry e = { .snapshot = inode_k.k->p.snapshot };
 					bch2_inode_opts_get_inode(c, &inode, &e.io_opts);
 
 					darray_push(&snapshot_opts->d, e);
-				}));
+				})));
 
 				snapshot_opts->cur_inum = k.k->p.inode;
 
-				return ret ?: bch_err_throw(c, transaction_restart_nested);
+				return bch_err_throw(c, transaction_restart_nested);
 			}
 
-			darray_for_each(snapshot_opts->d, i)
-				if (bch2_snapshot_is_ancestor(c, k.k->p.snapshot, i->snapshot)) {
-					*opts = i->io_opts;
-					return 0;
-				}
+			struct snapshot_io_opts_entry *i =
+				darray_find_p(snapshot_opts->d, i,
+					      bch2_snapshot_is_ancestor(c, k.k->p.snapshot, i->snapshot));
+			if (i) {
+				*opts = i->io_opts;
+				return 0;
+			}
 		}
 
 		*opts = snapshot_opts->fs_io_opts;
 	}
 
 	const struct bch_extent_rebalance *old;
-	if (k.k->type == KEY_TYPE_reflink_v &&
-	    (old = bch2_bkey_rebalance_opts(k))) {
+	if (mode == IO_OPTS_reflink &&
+	    (old = bch2_bkey_rebalance_opts(c, k))) {
 #define x(_name)								\
 		if (old->_name##_from_inode)					\
 			opts->_name		= old->_name;			\
@@ -374,10 +391,46 @@ static const char * const bch2_rebalance_state_strs[] = {
 #undef x
 };
 
-int bch2_set_rebalance_needs_scan_trans(struct btree_trans *trans, u64 inum)
+static u64 rebalance_scan_encode(struct rebalance_scan s)
+{
+	switch (s.type) {
+	case REBALANCE_SCAN_fs:
+		return 0;
+	case REBALANCE_SCAN_metadata:
+		return 1;
+	case REBALANCE_SCAN_device:
+		return s.dev + 32;
+	case REBALANCE_SCAN_inum:
+		return s.inum;
+	default:
+		BUG();
+	}
+}
+
+static struct rebalance_scan rebalance_scan_decode(u64 v)
+{
+	if (v == 0)
+		return (struct rebalance_scan) { .type = REBALANCE_SCAN_fs };
+	if (v == 1)
+		return (struct rebalance_scan) { .type = REBALANCE_SCAN_metadata };
+	if (v < BCACHEFS_ROOT_INO)
+		return (struct rebalance_scan) {
+			.type = REBALANCE_SCAN_device,
+			.dev =  v - 32,
+	};
+
+	return (struct rebalance_scan) {
+		.type = REBALANCE_SCAN_inum,
+		.inum = v,
+	};
+}
+
+int bch2_set_rebalance_needs_scan_trans(struct btree_trans *trans, struct rebalance_scan s)
 {
 	CLASS(btree_iter, iter)(trans, BTREE_ID_rebalance_work,
-				SPOS(inum, REBALANCE_WORK_SCAN_OFFSET, U32_MAX),
+				SPOS(rebalance_scan_encode(s),
+				     REBALANCE_WORK_SCAN_OFFSET,
+				     U32_MAX),
 				BTREE_ITER_intent);
 	struct bkey_s_c k = bkey_try(bch2_btree_iter_peek_slot(&iter));
 
@@ -394,16 +447,17 @@ int bch2_set_rebalance_needs_scan_trans(struct btree_trans *trans, u64 inum)
 	return bch2_trans_update(trans, &iter, &cookie->k_i, 0);
 }
 
-int bch2_set_rebalance_needs_scan(struct bch_fs *c, u64 inum)
+int bch2_set_rebalance_needs_scan(struct bch_fs *c, struct rebalance_scan s)
 {
 	CLASS(btree_trans, trans)(c);
 	return commit_do(trans, NULL, NULL, BCH_TRANS_COMMIT_no_enospc,
-			    bch2_set_rebalance_needs_scan_trans(trans, inum));
+			    bch2_set_rebalance_needs_scan_trans(trans, s));
 }
 
 int bch2_set_fs_needs_rebalance(struct bch_fs *c)
 {
-	return bch2_set_rebalance_needs_scan(c, 0);
+	return bch2_set_rebalance_needs_scan(c,
+				(struct rebalance_scan) { .type = REBALANCE_SCAN_fs });
 }
 
 static int bch2_clear_rebalance_needs_scan(struct btree_trans *trans, u64 inum, u64 cookie)
@@ -470,13 +524,15 @@ static int bch2_bkey_clear_needs_rebalance(struct btree_trans *trans,
 					   struct btree_iter *iter,
 					   struct bkey_s_c k)
 {
-	if (k.k->type == KEY_TYPE_reflink_v || !bch2_bkey_rebalance_opts(k))
+	struct bch_fs *c = trans->c;
+
+	if (k.k->type == KEY_TYPE_reflink_v || !bch2_bkey_rebalance_opts(c, k))
 		return 0;
 
 	struct bkey_i *n = errptr_try(bch2_bkey_make_mut(trans, iter, &k, 0));
 
-	extent_entry_drop(bkey_i_to_s(n),
-			  (void *) bch2_bkey_rebalance_opts(bkey_i_to_s_c(n)));
+	extent_entry_drop(c, bkey_i_to_s(n),
+			  (void *) bch2_bkey_rebalance_opts(c, bkey_i_to_s_c(n)));
 	return bch2_trans_commit(trans, NULL, NULL, BCH_TRANS_COMMIT_no_enospc);
 }
 
@@ -490,7 +546,6 @@ static int rebalance_set_data_opts(struct btree_trans *trans,
 	struct btree_iter *extent_iter = arg;
 	struct bch_fs *c = trans->c;
 
-	memset(data_opts, 0, sizeof(*data_opts));
 	data_opts->type			= BCH_DATA_UPDATE_rebalance;
 	data_opts->ptrs_rewrite		= bch2_bkey_ptrs_need_rebalance(c, opts, k);
 	data_opts->target		= opts->background_target;
@@ -647,7 +702,7 @@ root_err:
 noinline_for_stack
 static int do_rebalance_scan(struct moving_context *ctxt,
 			     struct per_snapshot_io_opts *snapshot_io_opts,
-			     u64 inum, u64 cookie, u64 *sectors_scanned)
+			     u64 scan_v, u64 cookie, u64 *sectors_scanned)
 {
 	struct btree_trans *trans = ctxt->trans;
 	struct bch_fs *c = trans->c;
@@ -658,7 +713,8 @@ static int do_rebalance_scan(struct moving_context *ctxt,
 
 	r->state = BCH_REBALANCE_scanning;
 
-	if (!inum) {
+	struct rebalance_scan s = rebalance_scan_decode(scan_v);
+	if (s.type == REBALANCE_SCAN_fs) {
 		r->scan_start	= BBPOS_MIN;
 		r->scan_end	= BBPOS_MAX;
 
@@ -670,16 +726,16 @@ static int do_rebalance_scan(struct moving_context *ctxt,
 			try(do_rebalance_scan_btree(ctxt, snapshot_io_opts, btree, 0,
 						    POS_MIN, SPOS_MAX));
 		}
-	} else {
-		r->scan_start	= BBPOS(BTREE_ID_extents, POS(inum, 0));
-		r->scan_end	= BBPOS(BTREE_ID_extents, POS(inum, U64_MAX));
+	} else if (s.type == REBALANCE_SCAN_inum) {
+		r->scan_start	= BBPOS(BTREE_ID_extents, POS(s.inum, 0));
+		r->scan_end	= BBPOS(BTREE_ID_extents, POS(s.inum, U64_MAX));
 
 		try(do_rebalance_scan_btree(ctxt, snapshot_io_opts, BTREE_ID_extents, 0,
 					    r->scan_start.pos, r->scan_end.pos));
 	}
 
 	try(commit_do(trans, NULL, NULL, BCH_TRANS_COMMIT_no_enospc,
-			bch2_clear_rebalance_needs_scan(trans, inum, cookie)));
+			bch2_clear_rebalance_needs_scan(trans, scan_v, cookie)));
 
 	*sectors_scanned += atomic64_read(&r->scan_stats.sectors_seen);
 	/*
@@ -1021,7 +1077,7 @@ int bch2_check_rebalance_work(struct bch_fs *c)
 	struct wb_maybe_flush last_flushed __cleanup(wb_maybe_flush_exit);
 	wb_maybe_flush_init(&last_flushed);
 
-	struct progress_indicator_state progress;
+	struct progress_indicator progress;
 	bch2_progress_init(&progress, c, BIT_ULL(BTREE_ID_rebalance_work));
 
 	int ret = 0;
