@@ -196,6 +196,8 @@ void bch2_snapshot_to_text(struct printbuf *out, struct bch_fs *c,
 		prt_str(out, "will_delete ");
 	if (BCH_SNAPSHOT_DELETED(s.v))
 		prt_str(out, "deleted ");
+	if (BCH_SNAPSHOT_NO_KEYS(s.v))
+		prt_str(out, "no_keys ");
 
 	prt_printf(out, "parent %10u children %10u %10u subvol %u tree %u",
 	       le32_to_cpu(s.v->parent),
@@ -694,6 +696,7 @@ static unsigned live_child(struct bch_fs *c, u32 start)
 	     id && id != start;
 	     id = bch2_snapshot_tree_next(t, id))
 		if (bch2_snapshot_is_leaf(c, id) &&
+		    bch2_snapshot_exists(c, id) &&
 		    !snapshot_list_has_id(&d->delete_leaves, id) &&
 		    !interior_delete_has_id(&d->delete_interior, id))
 			return id;
@@ -770,20 +773,22 @@ static int delete_dead_snapshot_keys_v1(struct btree_trans *trans)
 	struct bch_fs *c = trans->c;
 	struct snapshot_delete *d = &c->snapshot_delete;
 
-	for (d->pos.btree = 0; d->pos.btree < BTREE_ID_NR; d->pos.btree++) {
+	bch2_progress_init(&d->progress, c, btree_has_snapshots_mask);
+	d->progress.silent	= true;
+	d->version		= 1;
+
+	for (unsigned btree = 0; btree < BTREE_ID_NR; btree++) {
 		CLASS(disk_reservation, res)(c);
 		u64 prev_inum = 0;
 
-		d->pos.pos = POS_MIN;
-
-		if (!btree_type_has_snapshots(d->pos.btree))
+		if (!btree_type_has_snapshots(btree))
 			continue;
 
 		try(for_each_btree_key_commit(trans, iter,
-				d->pos.btree, POS_MIN,
+				btree, POS_MIN,
 				BTREE_ITER_prefetch|BTREE_ITER_all_snapshots, k,
 				&res.r, NULL, BCH_TRANS_COMMIT_no_enospc, ({
-			d->pos.pos = iter.pos;
+			progress_update_iter(trans, &d->progress, &iter);
 
 			if (skip_unrelated_snapshot_tree(trans, &iter, &prev_inum))
 				continue;
@@ -802,16 +807,11 @@ static int delete_dead_snapshot_keys_range(struct btree_trans *trans,
 					   struct bpos start, struct bpos end)
 {
 	struct bch_fs *c = trans->c;
-	struct snapshot_delete *d = &c->snapshot_delete;
-
-	d->pos.btree	= btree;
-	d->pos.pos	= POS_MIN;
 
 	return for_each_btree_key_max_commit(trans, iter,
 			btree, start, end,
 			BTREE_ITER_prefetch|BTREE_ITER_all_snapshots, k,
 			res, NULL, BCH_TRANS_COMMIT_no_enospc, ({
-		d->pos.pos = iter.pos;
 		bch2_disk_reservation_put(c, res);
 		delete_dead_snapshots_process_key(trans, &iter, k);
 	}));
@@ -824,8 +824,20 @@ static int delete_dead_snapshot_keys_v2(struct btree_trans *trans)
 	CLASS(disk_reservation, res)(c);
 	u64 prev_inum = 0;
 
+	bch2_progress_init(&d->progress, c, BIT_ULL(BTREE_ID_inodes));
+	d->progress.silent	= true;
+	d->version		= 2;
+
 	CLASS(btree_iter, iter)(trans, BTREE_ID_inodes, POS_MIN,
-			     BTREE_ITER_prefetch|BTREE_ITER_all_snapshots);
+				BTREE_ITER_prefetch|BTREE_ITER_all_snapshots);
+
+	/*
+	 * First, delete extents/dirents/xattrs
+	 *
+	 * If an extent/dirent/xattr is present in a given snapshot ID an inode
+	 * must also be present in that same snapshot ID, so we can use this to
+	 * greatly accelerate scanning:
+	 */
 
 	while (1) {
 		struct bkey_s_c k;
@@ -834,8 +846,7 @@ static int delete_dead_snapshot_keys_v2(struct btree_trans *trans)
 		if (!k.k)
 			break;
 
-		d->pos.btree	= iter.btree_id;
-		d->pos.pos	= iter.pos;
+		progress_update_iter(trans, &d->progress, &iter);
 
 		if (skip_unrelated_snapshot_tree(trans, &iter, &prev_inum))
 			continue;
@@ -854,14 +865,13 @@ static int delete_dead_snapshot_keys_v2(struct btree_trans *trans)
 		}
 	}
 
+	/* Then the inodes */
+
 	prev_inum = 0;
 	try(for_each_btree_key_commit(trans, iter,
 			BTREE_ID_inodes, POS_MIN,
 			BTREE_ITER_prefetch|BTREE_ITER_all_snapshots, k,
 			&res.r, NULL, BCH_TRANS_COMMIT_no_enospc, ({
-		d->pos.btree	= iter.btree_id;
-		d->pos.pos	= iter.pos;
-
 		if (skip_unrelated_snapshot_tree(trans, &iter, &prev_inum))
 			continue;
 
@@ -887,10 +897,9 @@ static int check_should_delete_snapshot(struct btree_trans *trans, struct bkey_s
 	struct bkey_s_c_snapshot s = bkey_s_c_to_snapshot(k);
 	unsigned live_children = 0;
 
-	if (BCH_SNAPSHOT_SUBVOL(s.v))
-		return 0;
-
-	if (BCH_SNAPSHOT_DELETED(s.v))
+	if (BCH_SNAPSHOT_SUBVOL(s.v) ||
+	    BCH_SNAPSHOT_NO_KEYS(s.v) ||
+	    BCH_SNAPSHOT_DELETED(s.v))
 		return 0;
 
 	guard(mutex)(&d->progress_lock);
@@ -1001,24 +1010,67 @@ static int bch2_fix_child_of_deleted_snapshot(struct btree_trans *trans,
 static void bch2_snapshot_delete_nodes_to_text(struct printbuf *out, struct snapshot_delete *d)
 {
 	prt_printf(out, "deleting from trees");
-	darray_for_each(d->deleting_from_trees, i)
+	darray_for_each_max(d->deleting_from_trees, i, 10)
 		prt_printf(out, " %u", *i);
 
+	if (d->deleting_from_trees.nr > 10)
+		prt_str(out, " (many)");
+	prt_newline(out);
+
 	prt_printf(out, "deleting leaves");
-	darray_for_each(d->delete_leaves, i)
+	darray_for_each_max(d->delete_leaves, i, 10)
 		prt_printf(out, " %u", *i);
+
+	if (d->delete_leaves.nr > 10)
+		prt_str(out, " (many)");
 	prt_newline(out);
 
 	prt_printf(out, "interior");
-	darray_for_each(d->delete_interior, i)
+	darray_for_each_max(d->delete_interior, i, 10)
 		prt_printf(out, " %u->%u", i->id, i->live_child);
+
+	if (d->delete_interior.nr > 10)
+		prt_str(out, " (many)");
 	prt_newline(out);
 }
 
+static int delete_dead_snapshots_locked(struct bch_fs *c)
+{
+	CLASS(btree_trans, trans)(c);
+
+	/*
+	 * For every snapshot node: If we have no live children and it's not
+	 * pointed to by a subvolume, delete it:
+	 */
+	try(for_each_btree_key(trans, iter, BTREE_ID_snapshots, POS_MIN, 0, k,
+		check_should_delete_snapshot(trans, k)));
+
+	struct snapshot_delete *d = &c->snapshot_delete;
+	if (!d->delete_leaves.nr && !d->delete_interior.nr)
+		return 0;
+
+	CLASS(printbuf, buf)();
+	bch2_snapshot_delete_nodes_to_text(&buf, d);
+	try(commit_do(trans, NULL, NULL, 0, bch2_trans_log_msg(trans, &buf)));
+
+	try(!bch2_request_incompat_feature(c, bcachefs_metadata_version_snapshot_deletion_v2)
+	    ? delete_dead_snapshot_keys_v2(trans)
+	    : delete_dead_snapshot_keys_v1(trans));
+
+	darray_for_each(d->delete_leaves, i)
+		try(commit_do(trans, NULL, NULL, 0,
+			bch2_snapshot_node_delete(trans, *i)));
+
+	darray_for_each(d->delete_interior, i)
+		try(commit_do(trans, NULL, NULL, 0,
+			bch2_snapshot_node_set_no_keys(trans, i->id)));
+
+	return 0;
+}
+
 int __bch2_delete_dead_snapshots(struct bch_fs *c)
 {
 	struct snapshot_delete *d = &c->snapshot_delete;
-	int ret = 0;
 
 	if (!mutex_trylock(&d->lock))
 		return 0;
@@ -1028,59 +1080,11 @@ int __bch2_delete_dead_snapshots(struct bch_fs *c)
 		return 0;
 	}
 
-	CLASS(btree_trans, trans)(c);
-
-	/*
-	 * For every snapshot node: If we have no live children and it's not
-	 * pointed to by a subvolume, delete it:
-	 */
 	d->running = true;
-	d->pos = BBPOS_MIN;
+	d->progress.pos = BBPOS_MIN;
 
-	ret = for_each_btree_key(trans, iter, BTREE_ID_snapshots, POS_MIN, 0, k,
-		check_should_delete_snapshot(trans, k));
-	if (!bch2_err_matches(ret, EROFS))
-		bch_err_msg(c, ret, "walking snapshots");
-	if (ret)
-		goto err;
+	int ret = delete_dead_snapshots_locked(c);
 
-	if (!d->delete_leaves.nr && !d->delete_interior.nr)
-		goto err;
-
-	{
-		CLASS(printbuf, buf)();
-		bch2_snapshot_delete_nodes_to_text(&buf, d);
-
-		ret = commit_do(trans, NULL, NULL, 0, bch2_trans_log_msg(trans, &buf));
-		if (ret)
-			goto err;
-	}
-
-	ret = !bch2_request_incompat_feature(c, bcachefs_metadata_version_snapshot_deletion_v2)
-		? delete_dead_snapshot_keys_v2(trans)
-		: delete_dead_snapshot_keys_v1(trans);
-	if (!bch2_err_matches(ret, EROFS))
-		bch_err_msg(c, ret, "deleting keys from dying snapshots");
-	if (ret)
-		goto err;
-
-	darray_for_each(d->delete_leaves, i) {
-		ret = commit_do(trans, NULL, NULL, 0,
-			bch2_snapshot_node_delete(trans, *i));
-		if (!bch2_err_matches(ret, EROFS))
-			bch_err_msg(c, ret, "deleting snapshot %u", *i);
-		if (ret)
-			goto err;
-	}
-	darray_for_each(d->delete_interior, i) {
-		ret = commit_do(trans, NULL, NULL, 0,
-			bch2_snapshot_node_set_no_keys(trans, i->id));
-		if (!bch2_err_matches(ret, EROFS))
-			bch_err_msg(c, ret, "deleting snapshot %u", i->id);
-		if (ret)
-			goto err;
-	}
-err:
 	scoped_guard(mutex, &d->progress_lock) {
 		darray_exit(&d->deleting_from_trees);
 		darray_exit(&d->delete_interior);
@@ -1136,8 +1140,11 @@ void bch2_snapshot_delete_status_to_text(struct printbuf *out, struct bch_fs *c)
 	}
 
 	scoped_guard(mutex, &d->progress_lock) {
+		prt_printf(out, "Snapshot deletion v%u\n", d->version);
+		prt_str(out, "Progress: ");
+		bch2_progress_to_text(out, &d->progress);
+		prt_newline(out);
 		bch2_snapshot_delete_nodes_to_text(out, d);
-		bch2_bbpos_to_text(out, d->pos);
 	}
 }
 
@@ -1258,13 +1265,9 @@ int bch2_snapshots_read(struct bch_fs *c)
 	 * initialized - so mark in reverse:
 	 */
 	CLASS(btree_trans, trans)(c);
-	int ret = for_each_btree_key_reverse(trans, iter, BTREE_ID_snapshots,
-				   POS_MAX, 0, k,
-			__bch2_mark_snapshot(trans, BTREE_ID_snapshots, 0, bkey_s_c_null, k, 0) ?:
-			bch2_check_snapshot_needs_deletion(trans, k));
-	bch_err_fn(c, ret);
-
-	return ret;
+	return for_each_btree_key_reverse(trans, iter, BTREE_ID_snapshots, POS_MAX, 0, k,
+		__bch2_mark_snapshot(trans, BTREE_ID_snapshots, 0, bkey_s_c_null, k, 0) ?:
+		bch2_check_snapshot_needs_deletion(trans, k));
 }
 
 void bch2_fs_snapshots_exit(struct bch_fs *c)
